@@ -2,6 +2,7 @@ import json
 from os import getenv
 from typing import Any, Dict, Optional, List, Callable, Coroutine
 from xpander_sdk import (
+    GraphItem,
     UserDetails,
     Memory,
     XpanderClient,
@@ -500,7 +501,7 @@ class ChainlitAdapter(SDKAdapter):
         return tool_calls
 
     @cl.step(type="tool", name="xpander.ai")
-    async def process_tool_calls(self, tool_calls: Dict[int, ToolCall], local_tools: Dict[str, Callable] = None) -> str | None:
+    async def process_tool_calls(self, tool_calls: Dict[int, ToolCall], on_tool_call: Optional[Callable], local_tools: Dict[str, Callable] = None) -> str | None:
         """
         Process tool calls by formatting their payloads and executing them.
 
@@ -581,10 +582,20 @@ class ChainlitAdapter(SDKAdapter):
                     tool_call_result.is_error = True
                     current_step.is_error = True
                 
+                
                 # report result to the memory
                 self.agent.add_tool_call_results(tool_call_results=[tool_call_result])
             else:
-                tool_call_result = self.agent.run_tool(tool=tool_call,payload_extension={"headers":{"x-xpander-source-node-type": SourceNodeType.ASSISTANT.value.lower()}})
+                
+                # report to parent window
+                graph_item = self.agent.graph.find_node_by_item_id(item_id=tool_call.name)
+                if graph_item and on_tool_call:
+                    await on_tool_call(tool_call=tool_call,graph_item=graph_item)
+                    
+                tool_call_result = self.agent.run_tool(tool=tool_call, payload_extension={"headers":{"x-xpander-source-node-type": SourceNodeType.ASSISTANT.value.lower()}})
+            
+            if tool_call_result.result == "STOP":
+                    current_step.name = f"Waiting for human approval ({tool_name})"
             
             if is_agent_end_tool:
                 current_step.output = tool_call_result.payload['bodyParams']['result']
